#ifndef __BKDDS_H__
#define __BKDDS_H__

#include <math.h>

#ifndef M_PI
# define M_PI           3.14159265358979323846  /* pi */
# define M_PI_2         1.57079632679489661923  /* pi/2 */
# define M_PI_4         0.78539816339744830962  /* pi/4 */
# define M_1_PI         0.31830988618379067154  /* 1/pi */
# define M_2_PI         0.63661977236758134308  /* 2/pi */
#endif
#define DDS_2PI (M_PI * 2)   /* 2 * Pi */
#define DDS_3PI2 (M_PI_2 * 3) /* 3/2 * pi */


#define SIN_TABLE_ORDER 8
#define SIN_TABLE_LEN (1<<SIN_TABLE_ORDER)
#define SIN_TABLE_MASK (SIN_TABLE_LEN-1)
#define ANG_INCR (SIN_TABLE_LEN / (M_PI * 2))

double sinus_table[SIN_TABLE_LEN+1];
int sinus_table_init = 0;

typedef struct {
    double sample_freq;
    double freq;
    double phase;
    double phase_step;
} dds_t;

#define dds_destroy(x)

static inline void dds_setphase(dds_t *dds, double phase) {
    dds->phase = phase * ANG_INCR; }
static inline double dds_getphase(dds_t *dds) {
    return dds->phase / ANG_INCR; }

static inline void dds_set_freq(dds_t *dds, double freq)
{
    dds->freq = freq;
    dds->phase_step = (freq / dds->sample_freq) * 2 * M_PI * ANG_INCR;    
}

inline static double interp(double phase)
{
    int x;
    double a, b;

    x = (int)(phase);
    b = phase - x;
    a = 1 - b;

    x &= SIN_TABLE_MASK;
    return (sinus_table[x] * a) 
	+ (sinus_table[(x+1)] * b);
}

static inline double dds_real(dds_t *dds)
{
    double sample;
    
    sample = interp(dds->phase);
    dds->phase += dds->phase_step;

    if (dds->phase > SIN_TABLE_LEN)
	dds->phase -= SIN_TABLE_LEN;

    return sample;
}

dds_t dds_init(double sample_freq, double freq, double phase)
{
    dds_t dds;
    int i;

    dds.sample_freq = sample_freq;
    dds.phase = phase * ANG_INCR;
    dds_set_freq(&dds, freq);

    /* Initialize sinus table */
    if (!sinus_table_init) {
	double incr = 1.0 / (double)SIN_TABLE_LEN;
	for (i = 0; i < SIN_TABLE_LEN; i++)
	    sinus_table[i] = sin(incr * i * DDS_2PI);
	sinus_table[SIN_TABLE_LEN] = sinus_table[0];
	sinus_table_init = 1;
    }

    return dds;
}

void dds_real_buf(dds_t *dds, double *buf, int count)
{
    int i;
    for (i = 0; i < count; i++)
	buf[i] = dds_real(dds);
}


#endif /* ! __BKDDS_H__ */
