
inline double modulate_sample(int lastwritepos, double lastfreq, double sample)
{
    double freq;
    double slope;

    /* Calculate modulator frequency at this point to lessen
       the calculations needed in the signal generator */
    freq = sample;
    freq *= DELTA_FREQ;
    freq += CARRIER;
    
    /* What we do here is calculate a linear "slope" from 
       the previous sample to this one. This is then used by 
       the modulator to gently increase/decrease the frequency
       with each sample without the need to recalculate
       the dds parameters. In fact this gives us a very
       efficient and pretty good interpolation filter. */
    slope = freq - lastfreq;
    slope /= CARRIER_PER_SIGNAL;
    slopebuf[lastwritepos] = slope;
    freqbuf[writepos] = freq;

    return freq;
}

/* Calculate the difference in 'us' between two timevals */
inline long int time_delta(struct timeval *tv1, struct timeval *tv2)
{
    long int us;
    us = (tv2->tv_sec - tv1->tv_sec)*1000000 + tv2->tv_usec;
    us = us - tv1->tv_usec;
    return us;
}


/* Generate the radio signal using the pre-calculated frequency information
 * in the freq buffer */
void fmsignal(SDL_Surface *sf)
{
    int x,y, t,i=CARRIER_PER_SIGNAL;
    register double freq;
    register double tmp;
    register double sum=0;
    long int us;
    SDL_Event event;
    struct timeval tv1, tv2;
    dds_t carrier;

    /* Prepare the oscillators */
    carrier = dds_init(SAMPLE_RATE, CARRIER, 0);
    
    gettimeofday(&tv1, 0);
    while (!theend) {
	if (SDL_PollEvent(&event) && event.type == SDL_MOUSEBUTTONDOWN)
	    return;
	
	SDL_LockSurface(sf);
	t=0;
	for (y=0; y < HEIGHT; y++) {
	    for (x=0; x < WIDTH; x++) {
		if (!i) {
		    i = CARRIER_PER_SIGNAL;
		    dds_set_freq(&carrier, freqbuf[readpos], 
				 slopebuf[readpos++]);
		    readpos &= BUFFER_SAMPLES_MASK;
		}
		i--;
		tmp = (dds_real(&carrier)+1) * 16777215;
		((Uint32*)sf->pixels)[t] = (int)tmp & 0x00ff0000;
		t++;
	    }
	    /* Compensate for horizontal blanking */
	    for (x=0; x < (HTOTAL-WIDTH); x++) {
		dds_real(&carrier);
	    }
	}
	/* Compensate for vertical blanking */
	for (x=0; x < (VTOTAL-HEIGHT)*HTOTAL; x++) {
	    if (!i) {
		i = CARRIER_PER_SIGNAL;
		dds_set_freq(&carrier, freqbuf[readpos], 
			     slopebuf[readpos++]);
		readpos &= BUFFER_SAMPLES_MASK;
	    }
	    i--;
	    dds_real(&carrier);
	}
	SDL_UnlockSurface(sf);
	
	gettimeofday(&tv2, 0);
	us = time_delta(&tv1, &tv2);
	us = FRAME_US - us;
	if (us > 0)
	    usleep(us);
	gettimeofday(&tv1, 0);
	SDL_Flip(sf);
    }
}
