int main(int ac, char **av)
{
    int i;
    
    /* Get the parameters */
    while ((i = getopt(ac, av, "w:W:h:H:r:c:f:")) != -1) {
	switch(i) {
	case 'f':
	    FILENAME = strdup(optarg);
	    break;
	case 'r':
	    SAMPLE_RATE = atoi(optarg);
	    break;
	case 'c':
	    CARRIER = atoi(optarg);
	    break;
	case 'w':
	    WIDTH = atoi(optarg);
	    break;
	case 'W':
	    HTOTAL = atoi(optarg);
	    break;
	case 'h':
	    HEIGHT = atoi(optarg);
	    break;
	case 'H':
	    VTOTAL = atoi(optarg);
	    break;
	}
    }
    
    /* Calculate needed constants */
    if (HTOTAL == 0) HTOTAL = WIDTH+1;
    if (VTOTAL == 0) VTOTAL = HEIGHT+1;
    PIXELS = WIDTH*HEIGHT;
    CARRIER_PER_SIGNAL = SAMPLE_RATE / SIGNAL_SAMPLES;
    FRAME_RATE = SAMPLE_RATE/HTOTAL/VTOTAL;
    FRAME_TICKS = 1000/FRAME_RATE; /* In ms */
    FRAME_US = 1000000/FRAME_RATE; /* In us */

    /* Decoded mp3 sounds */
    freqbuf = malloc(BUFFER_SAMPLES * sizeof(double));
    slopebuf = malloc(BUFFER_SAMPLES * sizeof(double));
    readpos = 0;
    writepos = 1;
    SDL_CreateThread(baseband_mod, 0);
    fprintf(stderr, "buffering audio...");
    while (readpos != writepos) 
	usleep(100000);
    fprintf(stderr, "done\n");

    /* Prepare SDL */
    SDL_Init(SDL_INIT_VIDEO);
    SDL_VideoInfo *vi = SDL_GetVideoInfo();
    printf("hwa: %d\n", vi->hw_available);
    printf("pix: %d\n", vi->vfmt->BitsPerPixel);

    SDL_Surface *sf = SDL_SetVideoMode(WIDTH, HEIGHT, 32, 
				       SDL_HWSURFACE|SDL_FULLSCREEN|
				       SDL_DOUBLEBUF);
    /*
    SDL_Overlay *ol = SDL_CreateYUVOverlay(WIDTH, HEIGHT, SDL_YV12_OVERLAY,
					   sf);
    */
    fprintf(stderr, "\n");
    fprintf(stderr, "SampleRate:   %3.2f MHz\n", (double)SAMPLE_RATE/1000000);
    fprintf(stderr, "Carrier:      %3.2f MHz\n", (double)CARRIER/1000000);
    fprintf(stderr, "CarrierPerA:  %3.2f\n", (double)CARRIER_PER_SIGNAL);
    fprintf(stderr, "FrameRate:    %3.2f\n", (double)FRAME_RATE);
    fprintf(stderr, "\n");

    /*
    printf("P2\n");
    printf("%d %d\n", WIDTH, HEIGHT);
    printf("255\n");
    */

    struct sched_param sp;
    sp.sched_priority = 10;
    sched_setscheduler(0, SCHED_RR, &sp);

    fmsignal(sf);

    /* Die */
    SDL_Quit();
}
