#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <SDL/SDL.h>
#include "dds2.h"

/* Image parameters */
#define WIDTH 1152
#define HEIGHT 864
//#define WIDTH 14832
//#define HEIGHT 14832

/* Carrier parameters */
#define SAMPLES 100         /* In MHz */
#define FREQ 8000         /* In kHz */
#define DELTA_FREQ 75000    /* In Hz */

/* Tone parameters */
#define TONE 900            /* In Hz */ 
#define TONE_SAMPLE 30000   /* In Hz */

/* Calculated stuff */
#define CARRIER (FREQ*1000)
#define HBLANK (HTOTAL-WIDTH)
#define VBLANK (VTOTAL-HEIGHT)
#define PIXELS (WIDTH*HEIGHT)
#define SAMPLE_RATE ((int)SAMPLES*1000000)
/* CARRIER_PER_TONE is how many carrier samples fall on one tone sample */
#define CARRIER_PER_TONE (SAMPLE_RATE / TONE_SAMPLE)
/* How many tone samples fall on a frame */
#define TONES_PER_FRAME ((float)PIXELS/(float)CARRIER_PER_TONE)

void fmsignal(dds_t *tone, dds_t *carrier, SDL_Surface *sf)
//void fmsignal(dds_t *tone, dds_t *carrier, SDL_Overlay *ol)
{
    int t, c, i=1;
    register double freq;
    register double tmp;
    register double sum=0;

    //    Uint8 *olpx = ol->pixels[0];

    //printf("PG: %d\n", SDL_GetTicks());
    //    SDL_LockSurface(sf);
    for (t=0; t < (int)TONES_PER_FRAME; t++) {
	tmp = dds_real(tone);
	freq = tmp*DELTA_FREQ + CARRIER;
	dds_set_freq(carrier, freq);
	for (c=0; c < CARRIER_PER_TONE; c++) {
	    tmp = (dds_real(carrier)+1) * 16777215;
	    //	    printf("%d\n", (int)tmp);
	    ((Uint32*)sf->pixels)[i] = (int)tmp & 0x00ff0000;
	    //	    ((Uint8*)sf->pixels)[i] = (Uint8)tmp;
	    //olpx[i] = (Uint8)tmp;
	    i++;
	}
    }
    //    SDL_UnlockSurface(sf);
    //    printf("PF: %d\n", SDL_GetTicks());
    usleep(3000);
    SDL_Flip(sf);
    //printf("AF: %d\n", SDL_GetTicks());

    /*
    SDL_Rect rect;
    rect.x = 0;
    rect.y = 0;
    rect.w = WIDTH;
    rect.h = HEIGHT;
    SDL_DisplayYUVOverlay(ol, &rect); 
    */
}

int main()
{
    dds_t tone;
    dds_t carrier;
    int i;

    /* Prepare the oscillators */
    tone = dds_init(TONE_SAMPLE, TONE, 0);
    carrier = dds_init(SAMPLE_RATE, CARRIER, 0);

    /* Prepare SDL */
    SDL_Init(SDL_INIT_VIDEO);
    SDL_VideoInfo *vi = SDL_GetVideoInfo();
    printf("hwa: %d\n", vi->hw_available);
    printf("pix: %d\n", vi->vfmt->BitsPerPixel);

    SDL_Surface *sf = SDL_SetVideoMode(WIDTH, HEIGHT, 32, 
				       SDL_HWSURFACE|SDL_FULLSCREEN|
				       SDL_DOUBLEBUF);
    /*
    SDL_Overlay *ol = SDL_CreateYUVOverlay(WIDTH, HEIGHT, SDL_YV12_OVERLAY,
					   sf);
    */
    fprintf(stderr, "\n");
    fprintf(stderr, "SampleRate:   %3.2f MHz\n", (double)SAMPLES);
    fprintf(stderr, "Carrier:      %3.2f MHz\n", (double)FREQ/1000);
    fprintf(stderr, "Tone:         %3.2f Hz\n", (double)TONE);
    fprintf(stderr, "CarrierPerT:  %3.2f\n", (double)CARRIER_PER_TONE);
    fprintf(stderr, "TonePerFrame: %3.2f\n", (double)TONES_PER_FRAME);
    fprintf(stderr, "\n");

    /*
    printf("P2\n");
    printf("%d %d\n", WIDTH, HEIGHT);
    printf("255\n");
    */

    for (i=0; i < 4*100; i++)
	fmsignal(&tone, &carrier, sf);
    

    /* Die */
    SDL_Quit();
}
