#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "dds2.h"

#define MHZ 121.5
#define WIDTH 1152
#define HEIGHT 864
#define HTOTAL 1568
#define VTOTAL 911
#define FREQ 13000          /* In kHz */
#define DELTA_FREQ 75000    /* In Hz */
#define TONE 870           /* In Hz */ 

#define CARRIER (FREQ*1000)
#define HBLANK (HTOTAL-WIDTH)
#define VBLANK (VTOTAL-HEIGHT)
#define PIXELS (WIDTH*HEIGHT)
#define SAMPLE_RATE ((int)MHZ*1000000)

void printtime()
{
    struct timeval tv;

    gettimeofday(&tv, 0);
    printf("%d.%d", tv.tv_sec, tv.tv_usec);
}

void amsignal(dds_t *tone, dds_t *carrier)
{
    int x,y,b;
    double tmp;

    /* Generate 1-frame AM signal */
    for (y = 0; y < HEIGHT; y++) {
	for (x = 0; x < WIDTH; x++) {
	    tmp = (dds_real(carrier)+1)*127;
	    tmp *= (dds_real(tone)/2 + 0.5);
	    printf("%d\n", (int)tmp);
	}
	/* Compensate for blanking */
	for (b = 0; b < HBLANK; b++) {
	    tmp = dds_real(carrier);
	}
    }
}

void fmsignal(dds_t *tone, dds_t *carrier)
{
    int x,y,b;
    double tmp;
    double freq;
    double maxfreq = -1;
    double minfreq = 999999999;
    double sum = 0;

    /* Generate 1-frame FM signal */
    for (y = 0; y < HEIGHT; y++) {
	for (x = 0; x < WIDTH; x++) {
	    tmp = dds_real(tone);
	    freq = tmp*DELTA_FREQ + CARRIER;
	    dds_set_freq(carrier, freq);
	    tmp = (dds_real(carrier)+1)*127;
	    sum += tmp;
	    printf("%d\n", (int)tmp);
	}
	/* Compensate for blanking */
	for (b = 0; b < HBLANK; b++) {
	    tmp = dds_real(carrier);
	}
    }
}

main()
{
    dds_t tone;
    dds_t carrier;

    /* Prepare the oscillators */
    tone = dds_init(SAMPLE_RATE, TONE, 0);
    carrier = dds_init(SAMPLE_RATE, CARRIER, 0);

    printf("P2\n");
    printf("%d %d\n", WIDTH, HEIGHT);
    printf("255\n");

    fmsignal(&tone, &carrier);
    
}
