#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <sys/select.h>
#include <unistd.h>
#include <sched.h>
#include <SDL/SDL.h>
#include "fmdds.h"

/* Image parameters */
int WIDTH = 2048;
int HEIGHT = 2048;
int PIXELS;
int HTOTAL = 0;
int VTOTAL = 0;

/* Signal parameters */
#define DELTA_FREQ 75000    /* In Hz */
#define SIGNAL_SAMPLES 110000   /* In Hz */
#define AUDIO_BW 15000     /* In Hz */
#define PILOT_FREQ 19000   /* In Hz */
#define STEREO_CARRIER 38000 /* In Hz */
int CARRIER = 35000000;
int SAMPLE_RATE = 73000000;
int CARRIER_PER_SIGNAL;
double FRAME_RATE;
int FRAME_TICKS; /* In ms */
int FRAME_US; /* In ms */

/* Misc */
char *FILENAME = "tst.mp3";
#define BUFFER_SAMPLES_SHIFT 16
#define BUFFER_SAMPLES (1<<BUFFER_SAMPLES_SHIFT)
#define BUFFER_SAMPLES_MASK ((1<<BUFFER_SAMPLES_SHIFT)-1)

volatile double *freqbuf; 
volatile double *slopebuf; 
volatile long int writepos, readpos;
volatile int theend;

/* The signal generation and some helpers are imported from here */
#include "fm.h"

int writelen(int maxlen)
{
    int rp = readpos;
    int len;

    if (rp < writepos)
	rp += BUFFER_SAMPLES;
    len = rp - writepos;
    /*
    printf("readpos:%d, writepos:%d, rp:%d, len:%d\n", readpos, writepos, 
	   rp, len);
    */
    return len > maxlen ? maxlen : len;
}

int baseband_mod(void *thing)
{
    FILE *pipe;
    int len;
    int i;
    char cmd[256];
    Sint16 tmpbuf[1024];
    double lastfreq = CARRIER;
    long int lastwritepos = writepos;
    double slope;
    dds_t pilot, stereo;
    double L, R, LpR, LmR;
    double sample;

    /* Prepare stereo carriers */
    pilot = dds_init(SIGNAL_SAMPLES, PILOT_FREQ, 0);
    stereo = dds_init(SIGNAL_SAMPLES, STEREO_CARRIER, 0);
    
    theend = 0;

    sprintf(cmd, "sox tst.mp3 -r %d -c 2 -s -2 -t raw - lowpass -2 %d", 
	    SIGNAL_SAMPLES, AUDIO_BW); 
    pipe = popen(cmd, "r");
    while (!theend) {
	len = writelen(1024);
	if (len > 0) {
	    len = fread(tmpbuf, 2, len, pipe);
	    theend = len == 0;
	    for (i=0; i < len;) {
		/* Get samples for both channels, and calculate the 
		 * mono (L+R) and the difference signal used to recreate
		 * the stereo data (L-R). */
		L = tmpbuf[i++] / 32767.0;
		R = tmpbuf[i++] / 32767.0;
		LpR = (L + R)/2;
		LmR = (L - R)/2;

		/* Create a composite sample consisting of the mono
		 * signal at baseband, a 19kHz pilot and a the difference
		 * signal DSB-SC modulated on a 38kHz carrier */
		sample = LpR;                      /* Mono signal */
		sample += 0.1*dds_real(&pilot);    /* Pilot */
		sample += LmR * dds_real(&stereo); /* DSB-SC stereo */

		/* Normalize so we get the signal within [-1, 1] */
		sample /= 2.1;

		/* Modulate and buffer the sample */
		lastfreq = modulate_sample(lastwritepos, lastfreq, sample);
		lastwritepos = writepos++;
		writepos %= BUFFER_SAMPLES;
	    }
	} else {
	    usleep(100000);
	}
    }
    printf("Yo, like end of line");
    exit(1);
}

#include "fmmain.h"
